@@ -52,7 +52,7 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#  include "config.h"
+#include "config.h"
 #endif
 
 #include "gsttheoraenc.h"
@@ -85,6 +85,25 @@ gst_border_mode_get_type (void)
   return border_mode_type;
 }
 
+#define GST_TYPE_MULTIPASS_MODE (gst_multipass_mode_get_type())
+static GType
+gst_multipass_mode_get_type (void)
+{
+  static GType multipass_mode_type = 0;
+  static const GEnumValue multipass_mode[] = {
+    {MULTIPASS_MODE_SINGLE_PASS, "Single pass", "single-pass"},
+    {MULTIPASS_MODE_FIRST_PASS, "First pass", "first-pass"},
+    {MULTIPASS_MODE_SECOND_PASS, "Second pass", "second-pass"},
+    {0, NULL, NULL},
+  };
+
+  if (!multipass_mode_type) {
+    multipass_mode_type =
+        g_enum_register_static ("GstTheoraEncMultipassMode", multipass_mode);
+  }
+  return multipass_mode_type;
+}
+
 /* taken from theora/lib/toplevel.c */
 static int
 _ilog (unsigned int v)
@@ -109,28 +128,32 @@ _ilog (unsigned int v)
 #define THEORA_DEF_CAP_OVERFLOW         TRUE
 #define THEORA_DEF_CAP_UNDERFLOW        FALSE
 #define THEORA_DEF_RATE_BUFFER          0
+#define THEORA_DEF_MULTIPASS_CACHE_FILE NULL
+#define THEORA_DEF_MULTIPASS_MODE       MULTIPASS_MODE_SINGLE_PASS
 enum
 {
-  ARG_0,
-  ARG_CENTER,
-  ARG_BORDER,
-  ARG_BITRATE,
-  ARG_QUALITY,
-  ARG_QUICK,
-  ARG_KEYFRAME_AUTO,
-  ARG_KEYFRAME_FREQ,
-  ARG_KEYFRAME_FREQ_FORCE,
-  ARG_KEYFRAME_THRESHOLD,
-  ARG_KEYFRAME_MINDISTANCE,
-  ARG_NOISE_SENSITIVITY,
-  ARG_SHARPNESS,
-  ARG_SPEEDLEVEL,
-  ARG_VP3_COMPATIBLE,
-  ARG_DROP_FRAMES,
-  ARG_CAP_OVERFLOW,
-  ARG_CAP_UNDERFLOW,
-  ARG_RATE_BUFFER,
-  /* FILL ME */
+  PROP_0,
+  PROP_CENTER,
+  PROP_BORDER,
+  PROP_BITRATE,
+  PROP_QUALITY,
+  PROP_QUICK,
+  PROP_KEYFRAME_AUTO,
+  PROP_KEYFRAME_FREQ,
+  PROP_KEYFRAME_FREQ_FORCE,
+  PROP_KEYFRAME_THRESHOLD,
+  PROP_KEYFRAME_MINDISTANCE,
+  PROP_NOISE_SENSITIVITY,
+  PROP_SHARPNESS,
+  PROP_SPEEDLEVEL,
+  PROP_VP3_COMPATIBLE,
+  PROP_DROP_FRAMES,
+  PROP_CAP_OVERFLOW,
+  PROP_CAP_UNDERFLOW,
+  PROP_RATE_BUFFER,
+  PROP_MULTIPASS_CACHE_FILE,
+  PROP_MULTIPASS_MODE
+      /* FILL ME */
 };
 
 /* this function does a straight granulepos -> timestamp conversion */
@@ -152,12 +175,6 @@ granulepos_to_timestamp (GstTheoraEnc * theoraenc, ogg_int64_t granulepos)
       theoraenc->info.fps_numerator);
 }
 
-static const GstElementDetails theora_enc_details =
-GST_ELEMENT_DETAILS ("Theora video encoder",
-    "Codec/Encoder/Video",
-    "encode raw YUV video to a theora stream",
-    "Wim Taymans <wim@fluendo.com>");
-
 static GstStaticPadTemplate theora_enc_sink_factory =
 GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
@@ -213,7 +230,10 @@ gst_theora_enc_base_init (gpointer g_class)
       gst_static_pad_template_get (&theora_enc_src_factory));
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&theora_enc_sink_factory));
-  gst_element_class_set_details (element_class, &theora_enc_details);
+  gst_element_class_set_details_simple (element_class,
+      "Theora video encoder", "Codec/Encoder/Video",
+      "encode raw YUV video to a theora stream",
+      "Wim Taymans <wim@fluendo.com>");
 }
 
 static void
@@ -226,88 +246,90 @@ gst_theora_enc_class_init (GstTheoraEncClass * klass)
   gobject_class->get_property = theora_enc_get_property;
   gobject_class->finalize = theora_enc_finalize;
 
-  g_object_class_install_property (gobject_class, ARG_CENTER,
+  g_object_class_install_property (gobject_class, PROP_CENTER,
       g_param_spec_boolean ("center", "Center",
           "ignored and kept for API compat only", TRUE,
           (GParamFlags) G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
-  g_object_class_install_property (gobject_class, ARG_BORDER,
+  g_object_class_install_property (gobject_class, PROP_BORDER,
       g_param_spec_enum ("border", "Border",
           "ignored and kept for API compat only",
           GST_TYPE_BORDER_MODE, BORDER_BLACK,
           (GParamFlags) G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   /* general encoding stream options */
-  g_object_class_install_property (gobject_class, ARG_BITRATE,
+  g_object_class_install_property (gobject_class, PROP_BITRATE,
       g_param_spec_int ("bitrate", "Bitrate", "Compressed video bitrate (kbps)",
           0, (1 << 24) - 1, THEORA_DEF_BITRATE,
-          (GParamFlags) G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
-  g_object_class_install_property (gobject_class, ARG_QUALITY,
+          (GParamFlags) G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS |
+          GST_PARAM_MUTABLE_PLAYING));
+  g_object_class_install_property (gobject_class, PROP_QUALITY,
       g_param_spec_int ("quality", "Quality", "Video quality", 0, 63,
           THEORA_DEF_QUALITY,
-          (GParamFlags) G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
-  g_object_class_install_property (gobject_class, ARG_QUICK,
+          (GParamFlags) G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS |
+          GST_PARAM_MUTABLE_PLAYING));
+  g_object_class_install_property (gobject_class, PROP_QUICK,
       g_param_spec_boolean ("quick", "Quick",
           "ignored and kept for API compat only", TRUE,
           (GParamFlags) G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
-  g_object_class_install_property (gobject_class, ARG_KEYFRAME_AUTO,
+  g_object_class_install_property (gobject_class, PROP_KEYFRAME_AUTO,
       g_param_spec_boolean ("keyframe-auto", "Keyframe Auto",
           "Automatic keyframe detection", THEORA_DEF_KEYFRAME_AUTO,
           (GParamFlags) G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
-  g_object_class_install_property (gobject_class, ARG_KEYFRAME_FREQ,
+  g_object_class_install_property (gobject_class, PROP_KEYFRAME_FREQ,
       g_param_spec_int ("keyframe-freq", "Keyframe frequency",
           "Keyframe frequency", 1, 32768, THEORA_DEF_KEYFRAME_FREQ,
           (GParamFlags) G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
-  g_object_class_install_property (gobject_class, ARG_KEYFRAME_FREQ_FORCE,
+  g_object_class_install_property (gobject_class, PROP_KEYFRAME_FREQ_FORCE,
       g_param_spec_int ("keyframe-force", "Keyframe force",
           "Force keyframe every N frames", 1, 32768,
           THEORA_DEF_KEYFRAME_FREQ_FORCE,
           (GParamFlags) G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
-  g_object_class_install_property (gobject_class, ARG_KEYFRAME_THRESHOLD,
+  g_object_class_install_property (gobject_class, PROP_KEYFRAME_THRESHOLD,
       g_param_spec_int ("keyframe-threshold", "Keyframe threshold",
           "ignored and kept for API compat only", 0, 32768, 80,
           (GParamFlags) G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
-  g_object_class_install_property (gobject_class, ARG_KEYFRAME_MINDISTANCE,
+  g_object_class_install_property (gobject_class, PROP_KEYFRAME_MINDISTANCE,
       g_param_spec_int ("keyframe-mindistance", "Keyframe mindistance",
           "ignored and kept for API compat only", 1, 32768, 8,
           (GParamFlags) G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
-  g_object_class_install_property (gobject_class, ARG_NOISE_SENSITIVITY,
+  g_object_class_install_property (gobject_class, PROP_NOISE_SENSITIVITY,
       g_param_spec_int ("noise-sensitivity", "Noise sensitivity",
           "ignored and kept for API compat only", 0, 32768, 1,
           (GParamFlags) G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
-  g_object_class_install_property (gobject_class, ARG_SHARPNESS,
+  g_object_class_install_property (gobject_class, PROP_SHARPNESS,
       g_param_spec_int ("sharpness", "Sharpness",
           "ignored and kept for API compat only", 0, 2, 0,
           (GParamFlags) G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
-  g_object_class_install_property (gobject_class, ARG_SPEEDLEVEL,
+  g_object_class_install_property (gobject_class, PROP_SPEEDLEVEL,
       g_param_spec_int ("speed-level", "Speed level",
           "Controls the amount of motion vector searching done while "
           "encoding.  This property requires libtheora version >= 1.0",
-          0, 2, THEORA_DEF_SPEEDLEVEL,
+          0, 3, THEORA_DEF_SPEEDLEVEL,
           (GParamFlags) G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
-  g_object_class_install_property (gobject_class, ARG_VP3_COMPATIBLE,
+  g_object_class_install_property (gobject_class, PROP_VP3_COMPATIBLE,
       g_param_spec_boolean ("vp3-compatible", "VP3 Compatible",
           "Disables non-VP3 compatible features."
           "  This property requires libtheora version >= 1.1",
           THEORA_DEF_VP3_COMPATIBLE,
           (GParamFlags) G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
-  g_object_class_install_property (gobject_class, ARG_DROP_FRAMES,
+  g_object_class_install_property (gobject_class, PROP_DROP_FRAMES,
       g_param_spec_boolean ("drop-frames", "VP3 Compatible",
           "Allow or disallow frame dropping."
           "  This property requires libtheora version >= 1.1",
           THEORA_DEF_DROP_FRAMES,
           (GParamFlags) G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
-  g_object_class_install_property (gobject_class, ARG_CAP_OVERFLOW,
+  g_object_class_install_property (gobject_class, PROP_CAP_OVERFLOW,
       g_param_spec_boolean ("cap-overflow", "VP3 Compatible",
           "Enable capping of bit reservoir overflows."
           "  This property requires libtheora version >= 1.1",
           THEORA_DEF_CAP_OVERFLOW,
           (GParamFlags) G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
-  g_object_class_install_property (gobject_class, ARG_CAP_UNDERFLOW,
+  g_object_class_install_property (gobject_class, PROP_CAP_UNDERFLOW,
       g_param_spec_boolean ("cap-underflow", "VP3 Compatible",
           "Enable capping of bit reservoir underflows."
           "  This property requires libtheora version >= 1.1",
           THEORA_DEF_CAP_UNDERFLOW,
           (GParamFlags) G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
-  g_object_class_install_property (gobject_class, ARG_RATE_BUFFER,
+  g_object_class_install_property (gobject_class, PROP_RATE_BUFFER,
       g_param_spec_int ("rate-buffer", "Rate Control Buffer",
           "Sets the size of the rate control buffer, in units of frames.  "
           "The default value of 0 instructs the encoder to automatically "
@@ -315,6 +337,15 @@ gst_theora_enc_class_init (GstTheoraEncClass * klass)
           "  This property requires libtheora version >= 1.1",
           0, 1000, THEORA_DEF_RATE_BUFFER,
           (GParamFlags) G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+  g_object_class_install_property (gobject_class, PROP_MULTIPASS_CACHE_FILE,
+      g_param_spec_string ("multipass-cache-file", "Multipass Cache File",
+          "Multipass cache file", THEORA_DEF_MULTIPASS_CACHE_FILE,
+          (GParamFlags) G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+  g_object_class_install_property (gobject_class, PROP_MULTIPASS_MODE,
+      g_param_spec_enum ("multipass-mode", "Multipass mode",
+          "Single pass or first/second pass", GST_TYPE_MULTIPASS_MODE,
+          THEORA_DEF_MULTIPASS_MODE,
+          (GParamFlags) G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   gstelement_class->change_state = theora_enc_change_state;
   GST_DEBUG_CATEGORY_INIT (theoraenc_debug, "theoraenc", 0, "Theora encoder");
@@ -352,6 +383,24 @@ gst_theora_enc_init (GstTheoraEnc * enc, GstTheoraEncClass * g_class)
   enc->cap_overflow = THEORA_DEF_CAP_OVERFLOW;
   enc->cap_underflow = THEORA_DEF_CAP_UNDERFLOW;
   enc->rate_buffer = THEORA_DEF_RATE_BUFFER;
+
+  enc->multipass_mode = THEORA_DEF_MULTIPASS_MODE;
+  enc->multipass_cache_file = THEORA_DEF_MULTIPASS_CACHE_FILE;
+}
+
+static void
+theora_enc_clear_multipass_cache (GstTheoraEnc * enc)
+{
+  if (enc->multipass_cache_fd) {
+    g_io_channel_shutdown (enc->multipass_cache_fd, TRUE, NULL);
+    g_io_channel_unref (enc->multipass_cache_fd);
+    enc->multipass_cache_fd = NULL;
+  }
+
+  if (enc->multipass_cache_adapter) {
+    gst_object_unref (enc->multipass_cache_adapter);
+    enc->multipass_cache_adapter = NULL;
+  }
 }
 
 static void
@@ -364,32 +413,75 @@ theora_enc_finalize (GObject * object)
     th_encode_free (enc->encoder);
   th_comment_clear (&enc->comment);
   th_info_clear (&enc->info);
+  g_free (enc->multipass_cache_file);
+
+  theora_enc_clear_multipass_cache (enc);
 
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
+static gboolean
+theora_enc_write_multipass_cache_beginning (GstTheoraEnc * enc, gboolean eos)
+{
+  GError *err = NULL;
+  GIOStatus stat;
+  gint bytes_read = 0;
+  gsize bytes_written = 0;
+  gchar *buf;
+
+  stat =
+      g_io_channel_seek_position (enc->multipass_cache_fd, 0, G_SEEK_SET, &err);
+  if (stat != G_IO_STATUS_ERROR) {
+    do {
+      bytes_read =
+          th_encode_ctl (enc->encoder, TH_ENCCTL_2PASS_OUT, &buf, sizeof (buf));
+      if (bytes_read > 0)
+        g_io_channel_write_chars (enc->multipass_cache_fd, buf, bytes_read,
+            &bytes_written, NULL);
+    } while (bytes_read > 0 && bytes_written > 0);
+
+  }
+
+  if (stat == G_IO_STATUS_ERROR || bytes_read < 0 || bytes_written < 0) {
+    if (eos)
+      GST_ELEMENT_WARNING (enc, RESOURCE, WRITE, (NULL),
+          ("Failed to seek to beginning of multipass cache file: %s",
+              err->message));
+    else
+      GST_ELEMENT_ERROR (enc, RESOURCE, WRITE, (NULL),
+          ("Failed to seek to beginning of multipass cache file: %s",
+              err->message));
+    g_error_free (err);
+
+    return FALSE;
+  }
+  return TRUE;
+}
+
 static void
 theora_enc_reset (GstTheoraEnc * enc)
 {
   ogg_uint32_t keyframe_force;
   int rate_flags;
 
+  GST_OBJECT_LOCK (enc);
+  enc->info.target_bitrate = enc->video_bitrate;
+  enc->info.quality = enc->video_quality;
+  enc->bitrate_changed = FALSE;
+  enc->quality_changed = FALSE;
+  GST_OBJECT_UNLOCK (enc);
+
   if (enc->encoder)
     th_encode_free (enc->encoder);
   enc->encoder = th_encode_alloc (&enc->info);
   /* We ensure this function cannot fail. */
   g_assert (enc->encoder != NULL);
-#ifdef TH_ENCCTL_SET_SPLEVEL
   th_encode_ctl (enc->encoder, TH_ENCCTL_SET_SPLEVEL, &enc->speed_level,
       sizeof (enc->speed_level));
-#endif
-#ifdef TH_ENCCTL_SET_VP3_COMPATIBLE
   th_encode_ctl (enc->encoder, TH_ENCCTL_SET_VP3_COMPATIBLE,
       &enc->vp3_compatible, sizeof (enc->vp3_compatible));
-#endif
 
   rate_flags = 0;
-#ifdef TH_ENCCTL_SET_RATE_FLAGS
   if (enc->drop_frames)
     rate_flags |= TH_RATECTL_DROP_FRAMES;
   if (enc->drop_frames)
@@ -398,21 +490,23 @@ theora_enc_reset (GstTheoraEnc * enc)
     rate_flags |= TH_RATECTL_CAP_UNDERFLOW;
   th_encode_ctl (enc->encoder, TH_ENCCTL_SET_RATE_FLAGS,
       &rate_flags, sizeof (rate_flags));
-#endif
 
-#ifdef TH_ENCCTL_SET_RATE_BUFFER
   if (enc->rate_buffer) {
     th_encode_ctl (enc->encoder, TH_ENCCTL_SET_RATE_BUFFER,
         &enc->rate_buffer, sizeof (enc->rate_buffer));
   } else {
     /* FIXME */
   }
-#endif
 
   keyframe_force = enc->keyframe_auto ?
       enc->keyframe_force : enc->keyframe_freq;
   th_encode_ctl (enc->encoder, TH_ENCCTL_SET_KEYFRAME_FREQUENCY_FORCE,
       &keyframe_force, sizeof (keyframe_force));
+
+  /* Get placeholder data */
+  if (enc->multipass_cache_fd
+      && enc->multipass_mode == MULTIPASS_MODE_FIRST_PASS)
+    theora_enc_write_multipass_cache_beginning (enc, FALSE);
 }
 
 static void
@@ -436,7 +530,7 @@ theora_enc_get_supported_formats (void)
   struct
   {
     th_pixel_fmt pixelformat;
-    char *fourcc;
+    const char *fourcc;
   } formats[] = {
     {
     TH_PF_420, "I420"}, {
@@ -548,8 +642,6 @@ theora_enc_sink_setcaps (GstPad * pad, GstCaps * caps)
   }
 
   enc->info.colorspace = TH_CS_UNSPECIFIED;
-  enc->info.target_bitrate = enc->video_bitrate;
-  enc->info.quality = enc->video_quality;
 
   /* as done in theora */
   enc->info.keyframe_granule_shift = _ilog (enc->keyframe_force - 1);
@@ -613,7 +705,7 @@ theora_buffer_from_packet (GstTheoraEnc * enc, ogg_packet * packet,
 
   /* the second most significant bit of the first data byte is cleared
    * for keyframes */
-  if ((packet->packet[0] & 0x40) == 0) {
+  if (packet->bytes > 0 && (packet->packet[0] & 0x40) == 0) {
     GST_BUFFER_FLAG_UNSET (buf, GST_BUFFER_FLAG_DELTA_UNIT);
   } else {
     GST_BUFFER_FLAG_SET (buf, GST_BUFFER_FLAG_DELTA_UNIT);
@@ -747,6 +839,12 @@ theora_enc_sink_event (GstPad * pad, GstEvent * event)
           enc->next_ts = next_time;
         }
       }
+      if (enc->initialised && enc->multipass_cache_fd
+          && enc->multipass_mode == MULTIPASS_MODE_FIRST_PASS)
+        theora_enc_write_multipass_cache_beginning (enc, TRUE);
+
+      theora_enc_clear_multipass_cache (enc);
+
       res = gst_pad_push_event (enc->srcpad, event);
       break;
     case GST_EVENT_FLUSH_STOP:
@@ -877,6 +975,79 @@ theora_enc_init_buffer (th_ycbcr_buffer buf, th_info * info, guint8 * data)
   }
 }
 
+static gboolean
+theora_enc_read_multipass_cache (GstTheoraEnc * enc)
+{
+  GstBuffer *cache_buf;
+  const guint8 *cache_data;
+  gsize bytes_read = 0, bytes_consumed = 0;
+  GIOStatus stat = G_IO_STATUS_NORMAL;
+  gboolean done = FALSE;
+
+  while (!done) {
+    if (gst_adapter_available (enc->multipass_cache_adapter) == 0) {
+      cache_buf = gst_buffer_new_and_alloc (512);
+      stat = g_io_channel_read_chars (enc->multipass_cache_fd,
+          (gchar *) GST_BUFFER_DATA (cache_buf), GST_BUFFER_SIZE (cache_buf),
+          &bytes_read, NULL);
+
+      if (bytes_read <= 0) {
+        gst_buffer_unref (cache_buf);
+        break;
+      } else {
+        GST_BUFFER_SIZE (cache_buf) = bytes_read;
+
+        gst_adapter_push (enc->multipass_cache_adapter, cache_buf);
+      }
+    }
+    if (gst_adapter_available (enc->multipass_cache_adapter) == 0)
+      break;
+
+    bytes_read =
+        MIN (gst_adapter_available (enc->multipass_cache_adapter), 512);
+
+    cache_data = gst_adapter_peek (enc->multipass_cache_adapter, bytes_read);
+
+    bytes_consumed =
+        th_encode_ctl (enc->encoder, TH_ENCCTL_2PASS_IN, (guint8 *) cache_data,
+        bytes_read);
+    done = bytes_consumed <= 0;
+    if (bytes_consumed > 0)
+      gst_adapter_flush (enc->multipass_cache_adapter, bytes_consumed);
+  }
+
+  if (stat == G_IO_STATUS_ERROR || (stat == G_IO_STATUS_EOF && bytes_read == 0)
+      || bytes_consumed < 0) {
+    GST_ELEMENT_ERROR (enc, RESOURCE, READ, (NULL),
+        ("Failed to read multipass cache file"));
+    return FALSE;
+  }
+  return TRUE;
+}
+
+static gboolean
+theora_enc_write_multipass_cache (GstTheoraEnc * enc)
+{
+  gchar *buf;
+  gint bytes_read;
+  gsize bytes_written = 0;
+
+  do {
+    bytes_read =
+        th_encode_ctl (enc->encoder, TH_ENCCTL_2PASS_OUT, &buf, sizeof (buf));
+    if (bytes_read > 0)
+      g_io_channel_write_chars (enc->multipass_cache_fd, buf, bytes_read,
+          &bytes_written, NULL);
+  } while (bytes_read > 0 && bytes_written > 0);
+
+  if (bytes_read < 0 || bytes_written < 0) {
+    GST_ELEMENT_ERROR (enc, RESOURCE, WRITE, (NULL),
+        ("Failed to write multipass cache file"));
+    return FALSE;
+  }
+  return TRUE;
+}
+
 static GstFlowReturn
 theora_enc_chain (GstPad * pad, GstBuffer * buffer)
 {
@@ -907,8 +1078,24 @@ theora_enc_chain (GstPad * pad, GstBuffer * buffer)
     return GST_FLOW_OK;
   }
 
-  /* see if we need to schedule a keyframe */
   GST_OBJECT_LOCK (enc);
+  if (enc->bitrate_changed) {
+    long int bitrate = enc->video_bitrate;
+
+    th_encode_ctl (enc->encoder, TH_ENCCTL_SET_BITRATE, &bitrate,
+        sizeof (long int));
+    enc->bitrate_changed = FALSE;
+  }
+
+  if (enc->quality_changed) {
+    long int quality = enc->video_quality;
+
+    th_encode_ctl (enc->encoder, TH_ENCCTL_SET_QUALITY, &quality,
+        sizeof (long int));
+    enc->quality_changed = FALSE;
+  }
+
+  /* see if we need to schedule a keyframe */
   force_keyframe = enc->force_keyframe;
   enc->force_keyframe = FALSE;
   GST_OBJECT_UNLOCK (enc);
@@ -1021,10 +1208,26 @@ theora_enc_chain (GstPad * pad, GstBuffer * buffer)
       enc->next_discont = TRUE;
     }
 
+    if (enc->multipass_cache_fd
+        && enc->multipass_mode == MULTIPASS_MODE_SECOND_PASS) {
+      if (!theora_enc_read_multipass_cache (enc)) {
+        ret = GST_FLOW_ERROR;
+        goto multipass_read_failed;
+      }
+    }
+
     res = th_encode_ycbcr_in (enc->encoder, ycbcr);
     /* none of the failure cases can happen here */
     g_assert (res == 0);
 
+    if (enc->multipass_cache_fd
+        && enc->multipass_mode == MULTIPASS_MODE_FIRST_PASS) {
+      if (!theora_enc_write_multipass_cache (enc)) {
+        ret = GST_FLOW_ERROR;
+        goto multipass_write_failed;
+      }
+    }
+
     ret = GST_FLOW_OK;
     while (th_encode_packetout (enc->encoder, 0, &op)) {
       GstClockTime next_time;
@@ -1045,6 +1248,16 @@ theora_enc_chain (GstPad * pad, GstBuffer * buffer)
   return ret;
 
   /* ERRORS */
+multipass_read_failed:
+  {
+    gst_buffer_unref (buffer);
+    return ret;
+  }
+multipass_write_failed:
+  {
+    gst_buffer_unref (buffer);
+    return ret;
+  }
 header_buffer_alloc:
   {
     gst_buffer_unref (buffer);
@@ -1086,6 +1299,33 @@ theora_enc_change_state (GstElement * element, GstStateChange transition)
       th_comment_init (&enc->comment);
       enc->packetno = 0;
       enc->force_keyframe = FALSE;
+
+      if (enc->multipass_mode >= MULTIPASS_MODE_FIRST_PASS) {
+        GError *err = NULL;
+
+        if (!enc->multipass_cache_file) {
+          ret = GST_STATE_CHANGE_FAILURE;
+          GST_ELEMENT_ERROR (enc, LIBRARY, SETTINGS, (NULL), (NULL));
+          return ret;
+        }
+        enc->multipass_cache_fd =
+            g_io_channel_new_file (enc->multipass_cache_file,
+            (enc->multipass_mode == MULTIPASS_MODE_FIRST_PASS ? "w" : "r"),
+            &err);
+
+        if (enc->multipass_mode == MULTIPASS_MODE_SECOND_PASS)
+          enc->multipass_cache_adapter = gst_adapter_new ();
+
+        if (!enc->multipass_cache_fd) {
+          ret = GST_STATE_CHANGE_FAILURE;
+          GST_ELEMENT_ERROR (enc, RESOURCE, OPEN_READ, (NULL),
+              ("Failed to open multipass cache file: %s", err->message));
+          g_error_free (err);
+          return ret;
+        }
+
+        g_io_channel_set_encoding (enc->multipass_cache_fd, NULL, NULL);
+      }
       break;
     case GST_STATE_CHANGE_PAUSED_TO_PLAYING:
       break;
@@ -1126,61 +1366,66 @@ theora_enc_set_property (GObject * object, guint prop_id,
   GstTheoraEnc *enc = GST_THEORA_ENC (object);
 
   switch (prop_id) {
-    case ARG_CENTER:
-    case ARG_BORDER:
-    case ARG_QUICK:
-    case ARG_KEYFRAME_THRESHOLD:
-    case ARG_KEYFRAME_MINDISTANCE:
-    case ARG_NOISE_SENSITIVITY:
-    case ARG_SHARPNESS:
+    case PROP_CENTER:
+    case PROP_BORDER:
+    case PROP_QUICK:
+    case PROP_KEYFRAME_THRESHOLD:
+    case PROP_KEYFRAME_MINDISTANCE:
+    case PROP_NOISE_SENSITIVITY:
+    case PROP_SHARPNESS:
       /* kept for API compat, but ignored */
       break;
-    case ARG_BITRATE:
+    case PROP_BITRATE:
+      GST_OBJECT_LOCK (enc);
       enc->video_bitrate = g_value_get_int (value) * 1000;
       enc->video_quality = 0;
+      enc->bitrate_changed = TRUE;
+      GST_OBJECT_UNLOCK (enc);
       break;
-    case ARG_QUALITY:
-      enc->video_quality = g_value_get_int (value);
-      enc->video_bitrate = 0;
+    case PROP_QUALITY:
+      GST_OBJECT_LOCK (enc);
+      if (GST_STATE (enc) >= GST_STATE_PAUSED && enc->video_quality == 0) {
+        GST_WARNING_OBJECT (object, "Can't change from bitrate to quality mode"
+            " while playing");
+      } else {
+        enc->video_quality = g_value_get_int (value);
+        enc->video_bitrate = 0;
+        enc->quality_changed = TRUE;
+      }
+      GST_OBJECT_UNLOCK (enc);
       break;
-    case ARG_KEYFRAME_AUTO:
+    case PROP_KEYFRAME_AUTO:
       enc->keyframe_auto = g_value_get_boolean (value);
       break;
-    case ARG_KEYFRAME_FREQ:
+    case PROP_KEYFRAME_FREQ:
       enc->keyframe_freq = g_value_get_int (value);
       break;
-    case ARG_KEYFRAME_FREQ_FORCE:
+    case PROP_KEYFRAME_FREQ_FORCE:
       enc->keyframe_force = g_value_get_int (value);
       break;
-    case ARG_SPEEDLEVEL:
-#ifdef TH_ENCCTL_SET_SPLEVEL
+    case PROP_SPEEDLEVEL:
       enc->speed_level = g_value_get_int (value);
-#endif
       break;
-    case ARG_VP3_COMPATIBLE:
-#ifdef TH_ENCCTL_SET_VP3_COMPATIBLE
+    case PROP_VP3_COMPATIBLE:
       enc->vp3_compatible = g_value_get_boolean (value);
-#endif
       break;
-    case ARG_DROP_FRAMES:
-#ifdef TH_ENCCTL_SET_RATE_FLAGS
+    case PROP_DROP_FRAMES:
       enc->drop_frames = g_value_get_boolean (value);
-#endif
       break;
-    case ARG_CAP_OVERFLOW:
-#ifdef TH_ENCCTL_SET_RATE_FLAGS
+    case PROP_CAP_OVERFLOW:
       enc->cap_overflow = g_value_get_boolean (value);
-#endif
       break;
-    case ARG_CAP_UNDERFLOW:
-#ifdef TH_ENCCTL_SET_RATE_FLAGS
+    case PROP_CAP_UNDERFLOW:
       enc->cap_underflow = g_value_get_boolean (value);
-#endif
       break;
-    case ARG_RATE_BUFFER:
-#ifdef TH_ENCCTL_SET_RATE_BUFFER
+    case PROP_RATE_BUFFER:
       enc->rate_buffer = g_value_get_int (value);
-#endif
+      break;
+    case PROP_MULTIPASS_CACHE_FILE:
+      enc->multipass_cache_file = g_value_dup_string (value);
+      break;
+    case PROP_MULTIPASS_MODE:
+      enc->multipass_mode = g_value_get_enum (value);
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -1195,60 +1440,70 @@ theora_enc_get_property (GObject * object, guint prop_id,
   GstTheoraEnc *enc = GST_THEORA_ENC (object);
 
   switch (prop_id) {
-    case ARG_CENTER:
+    case PROP_CENTER:
       g_value_set_boolean (value, TRUE);
       break;
-    case ARG_BORDER:
+    case PROP_BORDER:
       g_value_set_enum (value, BORDER_BLACK);
       break;
-    case ARG_BITRATE:
+    case PROP_BITRATE:
+      GST_OBJECT_LOCK (enc);
       g_value_set_int (value, enc->video_bitrate / 1000);
+      GST_OBJECT_UNLOCK (enc);
       break;
-    case ARG_QUALITY:
+    case PROP_QUALITY:
+      GST_OBJECT_LOCK (enc);
       g_value_set_int (value, enc->video_quality);
+      GST_OBJECT_UNLOCK (enc);
       break;
-    case ARG_QUICK:
+    case PROP_QUICK:
       g_value_set_boolean (value, TRUE);
       break;
-    case ARG_KEYFRAME_AUTO:
+    case PROP_KEYFRAME_AUTO:
       g_value_set_boolean (value, enc->keyframe_auto);
       break;
-    case ARG_KEYFRAME_FREQ:
+    case PROP_KEYFRAME_FREQ:
       g_value_set_int (value, enc->keyframe_freq);
       break;
-    case ARG_KEYFRAME_FREQ_FORCE:
+    case PROP_KEYFRAME_FREQ_FORCE:
       g_value_set_int (value, enc->keyframe_force);
       break;
-    case ARG_KEYFRAME_THRESHOLD:
+    case PROP_KEYFRAME_THRESHOLD:
       g_value_set_int (value, 80);
       break;
-    case ARG_KEYFRAME_MINDISTANCE:
+    case PROP_KEYFRAME_MINDISTANCE:
       g_value_set_int (value, 8);
       break;
-    case ARG_NOISE_SENSITIVITY:
+    case PROP_NOISE_SENSITIVITY:
       g_value_set_int (value, 1);
       break;
-    case ARG_SHARPNESS:
+    case PROP_SHARPNESS:
       g_value_set_int (value, 0);
       break;
-    case ARG_SPEEDLEVEL:
+    case PROP_SPEEDLEVEL:
       g_value_set_int (value, enc->speed_level);
       break;
-    case ARG_VP3_COMPATIBLE:
+    case PROP_VP3_COMPATIBLE:
       g_value_set_boolean (value, enc->vp3_compatible);
       break;
-    case ARG_DROP_FRAMES:
+    case PROP_DROP_FRAMES:
       g_value_set_boolean (value, enc->drop_frames);
       break;
-    case ARG_CAP_OVERFLOW:
+    case PROP_CAP_OVERFLOW:
       g_value_set_boolean (value, enc->cap_overflow);
       break;
-    case ARG_CAP_UNDERFLOW:
+    case PROP_CAP_UNDERFLOW:
       g_value_set_boolean (value, enc->cap_underflow);
       break;
-    case ARG_RATE_BUFFER:
+    case PROP_RATE_BUFFER:
       g_value_set_int (value, enc->rate_buffer);
       break;
+    case PROP_MULTIPASS_CACHE_FILE:
+      g_value_set_string (value, enc->multipass_cache_file);
+      break;
+    case PROP_MULTIPASS_MODE:
+      g_value_set_enum (value, enc->multipass_mode);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
