@@ -46,8 +46,6 @@
 GST_DEBUG_CATEGORY_STATIC (gst_ring_buffer_debug);
 #define GST_CAT_DEFAULT gst_ring_buffer_debug
 
-static void gst_ring_buffer_class_init (GstRingBufferClass * klass);
-static void gst_ring_buffer_init (GstRingBuffer * ringbuffer);
 static void gst_ring_buffer_dispose (GObject * object);
 static void gst_ring_buffer_finalize (GObject * object);
 
@@ -56,36 +54,8 @@ static void default_clear_all (GstRingBuffer * buf);
 static guint default_commit (GstRingBuffer * buf, guint64 * sample,
     guchar * data, gint in_samples, gint out_samples, gint * accum);
 
-static GstObjectClass *parent_class = NULL;
-
 /* ringbuffer abstract base class */
-GType
-gst_ring_buffer_get_type (void)
-{
-  static GType ringbuffer_type = 0;
-
-  if (G_UNLIKELY (!ringbuffer_type)) {
-    static const GTypeInfo ringbuffer_info = {
-      sizeof (GstRingBufferClass),
-      NULL,
-      NULL,
-      (GClassInitFunc) gst_ring_buffer_class_init,
-      NULL,
-      NULL,
-      sizeof (GstRingBuffer),
-      0,
-      (GInstanceInitFunc) gst_ring_buffer_init,
-      NULL
-    };
-
-    ringbuffer_type = g_type_register_static (GST_TYPE_OBJECT, "GstRingBuffer",
-        &ringbuffer_info, G_TYPE_FLAG_ABSTRACT);
-
-    GST_DEBUG_CATEGORY_INIT (gst_ring_buffer_debug, "ringbuffer", 0,
-        "ringbuffer class");
-  }
-  return ringbuffer_type;
-}
+G_DEFINE_ABSTRACT_TYPE (GstRingBuffer, gst_ring_buffer, GST_TYPE_OBJECT);
 
 static void
 gst_ring_buffer_class_init (GstRingBufferClass * klass)
@@ -96,10 +66,11 @@ gst_ring_buffer_class_init (GstRingBufferClass * klass)
   gobject_class = (GObjectClass *) klass;
   gstringbuffer_class = (GstRingBufferClass *) klass;
 
-  parent_class = g_type_class_peek_parent (klass);
+  GST_DEBUG_CATEGORY_INIT (gst_ring_buffer_debug, "ringbuffer", 0,
+      "ringbuffer class");
 
-  gobject_class->dispose = GST_DEBUG_FUNCPTR (gst_ring_buffer_dispose);
-  gobject_class->finalize = GST_DEBUG_FUNCPTR (gst_ring_buffer_finalize);
+  gobject_class->dispose = gst_ring_buffer_dispose;
+  gobject_class->finalize = gst_ring_buffer_finalize;
 
   gstringbuffer_class->clear_all = GST_DEBUG_FUNCPTR (default_clear_all);
   gstringbuffer_class->commit = GST_DEBUG_FUNCPTR (default_commit);
@@ -124,7 +95,8 @@ gst_ring_buffer_dispose (GObject * object)
 
   gst_caps_replace (&ringbuffer->spec.caps, NULL);
 
-  G_OBJECT_CLASS (parent_class)->dispose (G_OBJECT (ringbuffer));
+  G_OBJECT_CLASS (gst_ring_buffer_parent_class)->dispose (G_OBJECT
+      (ringbuffer));
 }
 
 static void
@@ -135,7 +107,8 @@ gst_ring_buffer_finalize (GObject * object)
   g_cond_free (ringbuffer->cond);
   g_free (ringbuffer->empty_seg);
 
-  G_OBJECT_CLASS (parent_class)->finalize (G_OBJECT (ringbuffer));
+  G_OBJECT_CLASS (gst_ring_buffer_parent_class)->finalize (G_OBJECT
+      (ringbuffer));
 }
 
 typedef struct
@@ -793,7 +766,7 @@ gst_ring_buffer_acquire (GstRingBuffer * buf, GstRingBufferSpec * spec)
 
   g_return_val_if_fail (GST_IS_RING_BUFFER (buf), FALSE);
 
-  GST_DEBUG_OBJECT (buf, "acquiring device");
+  GST_DEBUG_OBJECT (buf, "acquiring device %p", buf);
 
   GST_OBJECT_LOCK (buf);
   if (G_UNLIKELY (!buf->open))
@@ -864,7 +837,8 @@ acquire_failed:
 invalid_bps:
   {
     g_warning
-        ("invalid bytes_per_sample from acquire ringbuffer, fix the element");
+        ("invalid bytes_per_sample from acquire ringbuffer %p, fix the element",
+        buf);
     buf->acquired = FALSE;
     res = FALSE;
     goto done;
@@ -1415,8 +1389,8 @@ gst_ring_buffer_set_sample (GstRingBuffer * buf, guint64 sample)
 
   gst_ring_buffer_clear_all (buf);
 
-  GST_DEBUG_OBJECT (buf, "set sample to %llu, segbase %d", sample,
-      buf->segbase);
+  GST_DEBUG_OBJECT (buf, "set sample to %" G_GUINT64_FORMAT ", segbase %d",
+      sample, buf->segbase);
 }
 
 static void
@@ -1888,7 +1862,8 @@ gst_ring_buffer_read (GstRingBuffer * buf, guint64 sample, guchar * data,
       diff = segdone - readseg;
 
       GST_DEBUG
-          ("pointer at %d, sample %llu, read from %d-%d, to_read %d, diff %d, segtotal %d, segsize %d",
+          ("pointer at %d, sample %" G_GUINT64_FORMAT
+          ", read from %d-%d, to_read %d, diff %d, segtotal %d, segsize %d",
           segdone, sample, readseg, sampleoff, to_read, diff, segtotal,
           segsize);
 
@@ -1959,6 +1934,12 @@ gst_ring_buffer_prepare_read (GstRingBuffer * buf, gint * segment,
 
   g_return_val_if_fail (GST_IS_RING_BUFFER (buf), FALSE);
 
+  if (buf->callback == NULL) {
+    /* push mode, fail when nothing is started */
+    if (g_atomic_int_get (&buf->state) != GST_RING_BUFFER_STATE_STARTED)
+      return FALSE;
+  }
+
   g_return_val_if_fail (buf->data != NULL, FALSE);
   g_return_val_if_fail (segment != NULL, FALSE);
   g_return_val_if_fail (readptr != NULL, FALSE);
@@ -1966,12 +1947,6 @@ gst_ring_buffer_prepare_read (GstRingBuffer * buf, gint * segment,
 
   data = GST_BUFFER_DATA (buf->data);
 
-  if (buf->callback == NULL) {
-    /* push mode, fail when nothing is started */
-    if (g_atomic_int_get (&buf->state) != GST_RING_BUFFER_STATE_STARTED)
-      return FALSE;
-  }
-
   /* get the position of the pointer */
   segdone = g_atomic_int_get (&buf->segdone);
 
