@@ -443,7 +443,7 @@ gst_discoverer_info_get_video_streams (GstDiscovererInfo * info)
  * Finds all the #GstDiscovererContainerInfo contained in @info
  *
  * Returns: A #GList of matching #GstDiscovererStreamInfo. The caller should
- * free it with #g_list_free.
+ * free it with #gst_discoverer_stream_info_list_free.
  *
  * Since 0.10.31
  */
@@ -595,14 +595,13 @@ gst_discoverer_stream_info_get_misc (GstDiscovererStreamInfo * info)
  */
 
 GList *
-gst_discoverer_container_info_get_streams (GstDiscovererStreamInfo * info)
+gst_discoverer_container_info_get_streams (GstDiscovererContainerInfo * info)
 {
   GList *res = NULL, *tmp;
 
   g_return_val_if_fail (GST_IS_DISCOVERER_CONTAINER_INFO (info), NULL);
 
-  for (tmp = GST_DISCOVERER_CONTAINER_INFO (info)->streams; tmp;
-      tmp = tmp->next)
+  for (tmp = info->streams; tmp; tmp = tmp->next)
     res =
         g_list_append (res,
         gst_discoverer_stream_info_ref ((GstDiscovererStreamInfo *) tmp->data));
