@@ -170,6 +170,9 @@ gst_ffmpeg_pixfmt_to_caps (enum PixelFormat pix_fmt, AVCodecContext * context)
     case PIX_FMT_YUV420P:
       fmt = GST_MAKE_FOURCC ('I', '4', '2', '0');
       break;
+    case PIX_FMT_YUVA420P:
+      fmt = GST_MAKE_FOURCC ('A', '4', '2', '0');
+      break;
     case PIX_FMT_NV12:
       fmt = GST_MAKE_FOURCC ('N', 'V', '1', '2');
       break;
@@ -340,6 +343,25 @@ gst_ffmpeg_pixfmt_to_caps (enum PixelFormat pix_fmt, AVCodecContext * context)
     case PIX_FMT_YUV411P:
       fmt = GST_MAKE_FOURCC ('Y', '4', '1', 'B');
       break;
+    case PIX_FMT_Y800:{
+      GstCaps *tmp;
+
+      caps = gst_ff_vid_caps_new (context, "video/x-raw-yuv",
+          "format", GST_TYPE_FOURCC, GST_MAKE_FOURCC ('Y', '8', '0', '0'),
+          NULL);
+      tmp =
+          gst_ff_vid_caps_new (context, "video/x-raw-yuv", "format",
+          GST_TYPE_FOURCC, GST_MAKE_FOURCC ('Y', '8', ' ', ' '), NULL);
+      gst_caps_append (caps, tmp);
+      tmp = gst_ff_vid_caps_new (context, "video/x-raw-yuv",
+          "format", GST_TYPE_FOURCC, GST_MAKE_FOURCC ('G', 'R', 'E', 'Y'),
+          NULL);
+      gst_caps_append (caps, tmp);
+      break;
+    }
+    case PIX_FMT_Y16:
+      fmt = GST_MAKE_FOURCC ('Y', '1', '6', ' ');
+      break;
     case PIX_FMT_RGB565:
       bpp = depth = 16;
       endianness = G_BYTE_ORDER;
@@ -365,17 +387,10 @@ gst_ffmpeg_pixfmt_to_caps (enum PixelFormat pix_fmt, AVCodecContext * context)
     case PIX_FMT_AYUV4444:
       fmt = GST_MAKE_FOURCC ('A', 'Y', 'U', 'V');
       break;
-    case PIX_FMT_GRAY8:{
-      GstCaps *tmp;
-
+    case PIX_FMT_GRAY8:
       bpp = depth = 8;
       caps = gst_ff_vid_caps_new (context, "video/x-raw-gray",
           "bpp", G_TYPE_INT, bpp, "depth", G_TYPE_INT, depth, NULL);
-      tmp = gst_ff_vid_caps_new (context, "video/x-raw-yuv",
-          "format", GST_TYPE_FOURCC, GST_MAKE_FOURCC ('Y', '8', '0', '0'),
-          NULL);
-      gst_caps_append (caps, tmp);
-    }
       break;
     case PIX_FMT_GRAY16_L:
       bpp = depth = 16;
@@ -624,6 +639,9 @@ gst_ffmpeg_caps_to_pixfmt (const GstCaps * caps,
         case GST_MAKE_FOURCC ('I', '4', '2', '0'):
           context->pix_fmt = PIX_FMT_YUV420P;
           break;
+        case GST_MAKE_FOURCC ('A', '4', '2', '0'):
+          context->pix_fmt = PIX_FMT_YUVA420P;
+          break;
         case GST_MAKE_FOURCC ('N', 'V', '1', '2'):
           context->pix_fmt = PIX_FMT_NV12;
           break;
@@ -655,7 +673,12 @@ gst_ffmpeg_caps_to_pixfmt (const GstCaps * caps,
           context->pix_fmt = PIX_FMT_YUV444P;
           break;
         case GST_MAKE_FOURCC ('Y', '8', '0', '0'):
-          context->pix_fmt = PIX_FMT_GRAY8;
+        case GST_MAKE_FOURCC ('Y', '8', ' ', ' '):
+        case GST_MAKE_FOURCC ('G', 'R', 'E', 'Y'):
+          context->pix_fmt = PIX_FMT_Y800;
+          break;
+        case GST_MAKE_FOURCC ('Y', '1', '6', ' '):
+          context->pix_fmt = PIX_FMT_Y16;
           break;
       }
     }
@@ -840,6 +863,23 @@ gst_ffmpegcsp_avpicture_fill (AVPicture * picture,
       return size + 2 * size2;
       /* PIX_FMT_YVU420P = YV12: same as PIX_FMT_YUV420P, but
        *  with U and V plane swapped. Strides as in videotestsrc */
+    case PIX_FMT_YUVA420P:
+      stride = GST_ROUND_UP_4 (width);
+      h2 = ROUND_UP_X (height, pinfo->y_chroma_shift);
+      size = stride * h2;
+      w2 = DIV_ROUND_UP_X (width, pinfo->x_chroma_shift);
+      stride2 = GST_ROUND_UP_4 (w2);
+      h2 = DIV_ROUND_UP_X (height, pinfo->y_chroma_shift);
+      size2 = stride2 * h2;
+      picture->data[0] = ptr;
+      picture->data[1] = picture->data[0] + size;
+      picture->data[2] = picture->data[1] + size2;
+      picture->data[3] = picture->data[2] + size2;
+      picture->linesize[0] = stride;
+      picture->linesize[1] = stride2;
+      picture->linesize[2] = stride2;
+      picture->linesize[3] = stride;
+      return 2 * size + 2 * size2;
     case PIX_FMT_YVU410P:
     case PIX_FMT_YVU420P:
       stride = GST_ROUND_UP_4 (width);
@@ -918,14 +958,15 @@ gst_ffmpegcsp_avpicture_fill (AVPicture * picture,
       picture->linesize[0] = stride;
       return size;
     case PIX_FMT_UYVY411:
-      /* FIXME, probably not the right stride */
-      stride = GST_ROUND_UP_4 (width);
+      stride =
+          GST_ROUND_UP_4 (GST_ROUND_UP_4 (width) + GST_ROUND_UP_4 (width) / 2);
       size = stride * height;
       picture->data[0] = ptr;
       picture->data[1] = NULL;
       picture->data[2] = NULL;
-      picture->linesize[0] = width + width / 2;
-      return size + size / 2;
+      picture->linesize[0] = stride;
+      return size;
+    case PIX_FMT_Y800:
     case PIX_FMT_GRAY8:
       stride = GST_ROUND_UP_4 (width);
       size = stride * height;
@@ -934,6 +975,7 @@ gst_ffmpegcsp_avpicture_fill (AVPicture * picture,
       picture->data[2] = NULL;
       picture->linesize[0] = stride;
       return size;
+    case PIX_FMT_Y16:
     case PIX_FMT_GRAY16_L:
     case PIX_FMT_GRAY16_B:
       stride = GST_ROUND_UP_4 (width * 2);
